'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/*
 * Handles all data that does not need to persist
 * Essentially acts as a cache to reduce soundcloud api requests
 */

var DataManager = function () {
    function DataManager() {
        _classCallCheck(this, DataManager);

        this.data = {};
    }

    /* 
     * Returns the value associated with the key, if it exists.
     */


    _createClass(DataManager, [{
        key: 'get',
        value: function get(key) {
            return this.data[key];
        }

        /*
         * Adds a value to the data by key.
         */

    }, {
        key: 'set',
        value: function set(key, value) {
            this.data[key] = value;
        }

        /*
         * Removes a value from a specified key
         */

    }, {
        key: 'remove',
        value: function remove(key, value) {
            if (!this.exists(key)) {
                return false;
            }

            if (_typeof(this.get(key)) !== 'object') {
                return false;
            }

            this.get(key).remove(value);
            return true;
        }
    }, {
        key: 'removeById',
        value: function removeById(key, id) {
            if (!this.exists(key)) {
                return false;
            }

            if (_typeof(this.get(key)) !== 'object') {
                return false;
            }

            for (var i = 0; i < this.get(key).length; i++) {
                if (this.get(key)[i].id === id) {
                    this.get(key).splice(i, 1);
                    return true;
                }
            }
        }
    }, {
        key: 'find',
        value: function find(key, value) {
            if (!this.exists(key)) {
                return false;
            }

            if (_typeof(this.get(key)) !== 'object') {
                return false;
            }

            if (this.get(key).indexOf(value) !== -1) {
                return true;
            }

            return false;
        }

        /*
         * Finds the data associated with a key, and if it is an Array,
         * the value is added to the front of the array.
         */

    }, {
        key: 'unshift',
        value: function unshift(key, value) {
            if (!this.exists(key)) {
                return false;
            }

            if (_typeof(this.get(key)) !== 'object') {
                return false;
            }

            this.data[key].unshift(value);
            return true;
        }

        /*
         * Finds the data associated with a key, and if it is an Array,
         * the value is added to the end of the array.
         */

    }, {
        key: 'push',
        value: function push(key, value) {
            if (!this.exists(key)) {
                return false;
            }

            if (_typeof(this.get(key)) !== 'object') {
                return false;
            }

            this.data[key].push(value);
            return true;
        }
    }, {
        key: 'concat',
        value: function concat(key, array) {
            if (!this.exists(key)) {
                this.set(key, []);
            }

            if (_typeof(this.get(key)) !== 'object') {
                return false;
            }

            this.data[key] = this.data[key].concat(array);
            return true;
        }

        /*
         * Returns true if the key exists in the data, false otherwise.
         */

    }, {
        key: 'exists',
        value: function exists(key) {
            return this.data[key] !== undefined;
        }

        /*
         * Clears all stored data, which will force soundcloud api calls
         */

    }, {
        key: 'clear',
        value: function clear() {
            this.data = {};
        }

        /*
         * Clears stored data for a specified key, which will force a soundcloud api call
         */

    }, {
        key: 'bust',
        value: function bust(key) {
            delete this.data[key];
        }
    }]);

    return DataManager;
}();
"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Messenger = function () {
    function Messenger() {
        _classCallCheck(this, Messenger);

        this.queue = {};
        this.exists = this.queue.hasOwnProperty;
    }

    _createClass(Messenger, [{
        key: "subscribe",
        value: function subscribe(action, listener) {
            // Create the action's object if not yet created
            if (!this.exists.call(this.queue, action)) {
                this.queue[action] = [];
            }

            // add the listener to the queue
            var index = this.queue[action].push(listener) - 1;

            // provide a way to remove an action
            return {
                remove: function () {
                    delete this.queue[action][index];
                }.bind(this)
            };
        }
    }, {
        key: "publish",
        value: function publish(action, data) {
            // don't do anything if there are no listeners
            if (!this.exists.call(this.queue, action)) {
                return;
            }

            this.queue[action].forEach(function (actionItem) {
                actionItem(data != undefined ? data : {});
            });
        }
    }]);

    return Messenger;
}();
'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Music = function () {
    function Music() {
        _classCallCheck(this, Music);

        var self = this;

        soundManager.setup({
            url: '/path/to/swf-files/',
            flashVersion: 9, // optional: shiny features (default = 8)
            // optional: ignore Flash where possible, use 100% HTML5 mode
            // preferFlash: false,
            onready: function onready() {
                // Everything is loaded and ready
                self.isPlayerReady = true;
            }
        });

        this.currentlyPlaying = null;
        this.currentSoundObject = null;

        this.isRepeating = false;
        this.isShuffle = false;

        this.queue = [];

        window.messenger.subscribe('click-next-track', function (data) {
            if (this.currentlyPlaying !== null && this.currentlyPlaying.isPlaying) {
                this.playNext();
            }
        }.bind(this));
    }

    _createClass(Music, [{
        key: 'play',
        value: function play(track, onStartPlaying, onFinished) {
            if (!this.isPlayerReady) {
                return;
            }

            soundManager.stopAll();
            this.reset();

            this.currentSoundObject = soundManager.createSound({
                id: track.id,
                url: 'https://api.soundcloud.com/tracks/' + track.id + '/stream?client_id=173bf9df509c48cf53b70c83eaf5cbbd',
                title: track.title,
                volume: 50
            });

            this.setCurrentlyPlaying(track, onStartPlaying, onFinished);

            var self = this;

            this.currentSoundObject.play({
                onplay: function onplay() {
                    self.currentlyPlaying.isPlaying = true;

                    if (onStartPlaying != undefined) {
                        self.currentlyPlaying.onStartPlaying();
                    }

                    window.messenger.publish('music-state-change', { playing: true, id: self.currentlyPlaying.track.id });
                    window.messenger.publish('music-position-update', { position: this.position, duration: this.duration });
                },

                whileplaying: function whileplaying() {
                    window.messenger.publish('music-position-update', { position: this.position, duration: this.duration });
                },

                onfinish: function onfinish() {
                    if (self.isRepeating) {
                        self.play(track, onStartPlaying, onFinished);
                    } else {
                        if (onFinished !== undefined) {
                            self.currentlyPlaying.onFinished();
                        }

                        self.playNext(onStartPlaying, onFinished);
                    }
                }
            });
        }
    }, {
        key: 'playNext',
        value: function playNext(onStartPlaying, onFinished) {
            if (this.queue.length > 1) {
                this.shiftQueue();

                this.play(this.queue[0], onStartPlaying, onFinished);
            }
        }
    }, {
        key: 'pause',
        value: function pause() {
            if (this.currentlyPlaying !== null) {
                if (this.currentlyPlaying.isPlaying) {
                    this.currentlyPlaying.isPlaying = false;
                    soundManager.pauseAll();
                    return true;
                }
            }

            return false;
        }
    }, {
        key: 'resume',
        value: function resume() {
            if (this.currentlyPlaying !== null) {
                if (!this.currentlyPlaying.isPlaying) {
                    if (this.currentSoundObject !== null) {
                        this.currentlyPlaying.isPlaying = true;
                        this.currentSoundObject.resume();
                        return true;
                    }
                }
            }

            return false;
        }
    }, {
        key: 'setCurrentlyPlaying',
        value: function setCurrentlyPlaying(track, onStartPlaying, onFinished) {
            if (onStartPlaying === undefined) {
                onStartPlaying = function onStartPlaying() {};
            }

            if (onFinished === undefined) {
                onFinished = function onFinished() {};
            }

            this.currentlyPlaying = {
                track: track,
                onStartPlaying: onStartPlaying,
                onFinished: onFinished,
                isPlaying: false
            };
        }
    }, {
        key: 'updatePosition',
        value: function updatePosition(newPosition) {
            if (this.currentlyPlaying !== null && this.currentlyPlaying.isPlaying) {
                soundManager.setPosition(this.currentSoundObject.id, newPosition);

                window.messenger.publish('music-position-update', {
                    position: this.currentSoundObject.position,
                    duration: this.currentSoundObject.duration
                });
            }
        }
    }, {
        key: 'toggleRepeat',
        value: function toggleRepeat() {
            this.isRepeating = !this.isRepeating;
        }
    }, {
        key: 'toggleShuffle',
        value: function toggleShuffle() {
            this.isShuffle = !this.isShuffle;
            // send it to the set method for shuffling

            if (this.isShuffle) {
                this.setQueue(this.queue);
            } else {
                // publishing this action will cause setQueue to be called with the
                // non-shuffled music list, as retrieved from the musicList component
                window.messenger.publish('music-list-update-music-track-list', {});
            }
        }
    }, {
        key: 'setVolume',
        value: function setVolume(volume) {
            soundManager.setVolume(volume);
        }
    }, {
        key: 'reset',
        value: function reset() {
            if (this.currentSoundObject !== null) {
                this.currentSoundObject.clearOnPosition();
            }
            this.currentSoundObject = null;
            this.currentlyPlaying = null;
        }
    }, {
        key: 'convertDuration',
        value: function convertDuration(millis) {
            var hours = Math.floor(millis / 36e5),
                mins = Math.floor(millis % 36e5 / 6e4),
                secs = Math.floor(millis % 6e4 / 1000);
            return (hours > 0 ? hours + ":" : '') + (mins < 10 && hours > 0 ? '0' : '') + mins + ":" + (secs < 10 ? '0' : '') + secs;
        }

        /* Queue Methods */

    }, {
        key: 'setQueue',
        value: function setQueue(tracks) {
            if (this.isShuffle) {
                // removing the first element before shuffling and then adding it back
                // to the front ensures that the currently playing track remains #1
                var firstTrack = tracks.shift();
                this.shuffleQueue(tracks);
                tracks.unshift(firstTrack);
            }

            this.queue = tracks;
            window.messenger.publish('queue-order-updated', { tracks: this.queue });
        }
    }, {
        key: 'getQueue',
        value: function getQueue() {
            return this.queue;
        }
    }, {
        key: 'shiftQueue',
        value: function shiftQueue() {
            this.queue.shift();
            window.messenger.publish('queue-order-updated', { tracks: this.queue });
        }
    }, {
        key: 'removeFromQueue',
        value: function removeFromQueue(id, index) {
            if (this.getQueue()[index].id === id) {
                // if the currently playing song is removed from the queue,
                // the current song should be skipped
                if (this.currentlyPlaying !== null && this.currentlyPlaying.track.id === id) {
                    this.playNext();
                } else {
                    this.getQueue().splice(index, 1);
                    window.messenger.publish('queue-order-updated', { tracks: this.queue });
                }
            }
        }
    }, {
        key: 'shuffleQueue',
        value: function shuffleQueue() {
            var currentIndex = this.queue.length,
                temporaryValue,
                randomIndex;

            // While there remain elements to shuffle...
            while (0 !== currentIndex) {
                // Pick a remaining element...
                randomIndex = Math.floor(Math.random() * currentIndex);
                currentIndex -= 1;

                // And swap it with the current element.
                temporaryValue = this.queue[currentIndex];
                this.queue[currentIndex] = this.queue[randomIndex];
                this.queue[randomIndex] = temporaryValue;
            }
        }
    }]);

    return Music;
}();
'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var StorageManager = function () {
    function StorageManager(callback) {
        _classCallCheck(this, StorageManager);

        this.fs = node.fs;
        this.path = node.electron.remote.app.getPath('appData');

        this.directory = 'Soundcloud-Client';
        this.file = 'data';

        this.data = {};

        this.initializeStorage();

        this.readFromStorage(function (data) {
            this.data = data;
            callback();
        }.bind(this));
    }

    _createClass(StorageManager, [{
        key: 'initializeStorage',
        value: function initializeStorage() {
            var directoryPath = this.path + '/' + this.directory;
            var filePath = directoryPath + '/' + this.file;

            // initialize the directory
            if (!this.directoryOrFileExists(directoryPath)) {
                this.fs.mkdirSync(directoryPath);
            }
            // initialize the file
            if (!this.directoryOrFileExists(filePath)) {
                this.writeToStorage();
            }
        }
    }, {
        key: 'directoryOrFileExists',
        value: function directoryOrFileExists(path) {
            try {
                this.fs.accessSync(path, this.fs.F_OK);
                return true;
            } catch (err) {
                console.log(err);
                return false;
            }
        }
    }, {
        key: 'get',
        value: function get(property) {
            return this.propertyExists(property) ? this.data[property] : null;
        }
    }, {
        key: 'set',
        value: function set(property, value) {
            this.data[property] = value;
            this.writeToStorage();
        }
    }, {
        key: 'remove',
        value: function remove(property) {
            delete this.data[property];
            this.writeToStorage();
        }
    }, {
        key: 'propertyExists',
        value: function propertyExists(property) {
            return this.data.hasOwnProperty(property) ? true : false;
        }
    }, {
        key: 'writeToStorage',
        value: function writeToStorage() {
            this.fs.writeFileSync(this.path + '/Soundcloud-Client/data', JSON.stringify(this.data), 'utf8', function (err) {
                if (err) {
                    console.log(err);
                }
            });
        }
    }, {
        key: 'readFromStorage',
        value: function readFromStorage(callback) {
            this.fs.readFile(this.path + '/Soundcloud-Client/data', 'utf8', function (err, data) {

                if (err) {
                    console.log(err);
                    return {};
                }

                callback(JSON.parse(data));
            });
        }
    }]);

    return StorageManager;
}();
'use strict';

Array.prototype.remove = function () {
    var what,
        a = arguments,
        L = a.length,
        ax;
    while (L && this.length) {
        what = a[--L];
        while ((ax = this.indexOf(what)) !== -1) {
            this.splice(ax, 1);
        }
    }
    return this;
};

function isEmpty(object) {
    return Object.keys(object).length === 0 && object.constructor === Object;
}

var node = {
    fs: require('fs'),
    electron: require('electron'),
    renderer: require('electron').ipcRenderer,
    request: require('request')
};

node.renderer.on('click-play-pause', function () {
    window.messenger.publish('click-play-pause', {});
});

node.renderer.on('click-next-track', function () {
    window.messenger.publish('click-next-track', {});
});
'use strict';

var config = {
    // soundcloud
    base_url: 'https://api.soundcloud.com',
    base_urlv2: 'https://api-v2.soundcloud.com',
    redirect_uri: 'my-app://callback.html',
    client_id: '173bf9df509c48cf53b70c83eaf5cbbd',
    client_secret: '7ddbd6fcdc2d313abfb65758c751486e',
    connection_url: 'https://api.soundcloud.com/connect?client_id=173bf9df509c48cf53b70c83eaf5cbbd&redirect_uri=my-app%3A%2F%2Fcallback.html&response_type=code',

    // other stuff
    music_download_path: node.electron.remote.app.getPath('appData') + '/Soundcloud-Client/music'
};

config.soundcloud_urls = {
    me: {
        base: '/me',
        params: ['oauth_token'],
        next_href: null
    },

    stream: {
        base: '/me/activities',
        params: ['limit', 'oauth_token'],
        next_href: null,
        callback: function callback(response, _callback) {
            var activities = response.collection;
            var tracks = [];

            activities.forEach(function (activity) {
                if (activity.origin !== null) {
                    if (activity.origin.kind === 'track') {
                        tracks.push(activity.origin);
                    }
                }
            });

            config.soundcloud_urls['stream'].next_href = response.next_href;
            window.dataManager.concat('stream', tracks);

            _callback(tracks);
        }
    },

    user_by_id: {
        base: function base(user_id) {
            return '/users/' + user_id;
        },
        params: ['client_id'],
        next_href: null
    },

    // Playlists
    playlists: {
        base: function base(user_id) {
            return '/users/' + user_id + '/playlists/liked_and_owned';
        },
        params: ['oauth_token'],
        next_href: null
    },

    playlist: {
        base: function base(id) {
            return '/playlists/' + id;
        },
        params: ['client_id'],
        next_href: null
    },

    tracks_by_ids: {
        base: '/tracks',
        params: ['ids', 'client_id'],
        next_href: null
    },

    // Tracks
    my_tracks: {
        base: '/me/tracks',
        params: ['limit', 'offset', 'oauth_token'],
        next_href: null
    },

    // Likes
    liked_tracks: {
        base: function base(user_id) {
            return '/users/' + user_id + '/likes';
        },
        params: ['limit', 'offset', 'linked_partitioning', 'oauth_token'],
        next_href: null,
        callback: function callback(response, _callback2) {
            var tracks = [];

            response.collection.forEach(function (element) {
                if (element.track !== undefined) {
                    tracks.push(element.track);
                }
            });

            config.soundcloud_urls['liked_tracks'].next_href = response.next_href;
            window.dataManager.concat('liked_tracks', tracks);

            _callback2(tracks);
        }
    },

    liked_track_ids: {
        base: '/me/favorites/ids',
        params: ['oauth_token', 'limit', 'linked_partitioning', 'page_number', 'page_size'],
        next_href: null
    },

    track_repost_ids: {
        base: '/e1/me/track_reposts/ids',
        params: ['oauth_token', 'limit', 'linked_partitioning', 'page_number', 'page_size'],
        next_href: null
    },

    top_50: {
        base: '/charts',
        params: ['kind', 'genre', 'client_id', 'limit', 'offset', 'linked_partitioning'],
        next_href: null
    },

    // Search
    search: {
        base: '/search/tracks',
        params: ['q', 'facet', 'limit', 'client_id', 'linked_partitioning'],
        next_href: null
    },

    autocomplete: {
        base: '/search/autocomplete',
        params: ['q', 'queries_limit', 'results_limit', 'client_id', 'limit', 'offset', 'linked_partitioning'],
        next_href: null
    },

    // Notifications
    notifications: {
        base: '/activities',
        params: ['limit', 'offset', 'linked_partitioning', 'oauth_token'],
        next_href: null
    }

    // Toggling

};

config.soundcloud_storage = {
    likes: 'liked_tracks',
    stream: 'stream'
};
'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Downloader = function () {
    function Downloader() {
        _classCallCheck(this, Downloader);

        this.path = config.music_download_path;

        this.initializeStorage();
    }

    _createClass(Downloader, [{
        key: 'initializeStorage',
        value: function initializeStorage() {
            // initialize the directory
            if (!this.directoryOrFileExists(this.path)) {
                node.fs.mkdirSync(this.path);
            }
        }
    }, {
        key: 'directoryOrFileExists',
        value: function directoryOrFileExists(path) {
            try {
                node.fs.accessSync(path, node.fs.F_OK);
                return true;
            } catch (err) {
                return false;
            }
        }
    }, {
        key: 'downloadTrack',
        value: function downloadTrack(id, title, dest) {
            var url = 'https://api.soundcloud.com/tracks/' + id + '/stream?client_id=' + config.client_id;
            dest = this.path + '/' + title + '.mp3';

            var file = node.fs.createWriteStream(dest);
            var request = node.request.get(url);

            request.pipe(file);

            request.on('error', function (err) {
                // Handle errors
                fs.unlink(dest); // Delete the file async. (But we don't check the result)
            });
            request.on('response', function (response) {
                window.messenger.publish('download-file-size', { size: response.headers['content-length'] });
            });
            request.on('data', function (data) {
                window.messenger.publish('download-chunk-length', { size: data.length });
            });

            file.on('finish', function () {
                window.messenger.publish('download-complete', {});
                file.close(); // close() is async, call cb after close completes.
            });

            request.end();
        }
    }]);

    return Downloader;
}();

window.downloader = new Downloader();
'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var OAuthToken = function () {
    function OAuthToken(callback) {
        _classCallCheck(this, OAuthToken);

        this.request = new Request();

        // called when the logout button is clicked
        window.messenger.subscribe('logout', function (data) {
            this.clear();
        }.bind(this));

        this.token = {
            access_token: undefined,
            expires_in: undefined,
            refresh_token: undefined,
            scope: undefined
        };

        // first try to get the user token from storage
        this.token = window.storageManager.get('token');

        var token_type = 'authorization';

        // if the token doesn't exist then we need to authenticate the user
        // if the token already exists, just refresh it
        if (this.token !== null) {
            token_type = 'refresh';

            this.getToken(token_type, function (response) {
                this.token = response;
                callback();
            }.bind(this));
        } else {
            this.getAuthorizationCode(function (authorizationCode) {
                this.authorizationCode = authorizationCode;

                this.getToken(token_type, function (response) {
                    this.token = response;
                    callback();
                }.bind(this));
            }.bind(this));
        }
    }

    _createClass(OAuthToken, [{
        key: 'get',
        value: function get(key) {
            return this.token[key];
        }
    }, {
        key: 'clear',
        value: function clear() {
            window.storageManager.set('token', null);
            this.token = null;
            window.location.href = config.connection_url;
        }
    }, {
        key: 'getAuthorizationCode',
        value: function getAuthorizationCode(callback) {
            var url = window.location.href;
            var authorizationCode = url.substring(url.indexOf('=') + 1);

            // the user has already authorized the application
            if (url.indexOf('=') !== -1) {
                callback(authorizationCode);

                // if we don't yet have the authentication code,
                // then the user needs to connect and authorize the application
            } else {
                window.location.href = config.connection_url;
            }
        }
    }, {
        key: 'getToken',
        value: function getToken(type, callback) {
            var data = {},
                url = '/oauth2/token';

            // initial exchange of the token
            if (type === 'authorization') {
                data = {
                    'client_id': config.client_id,
                    'client_secret': config.client_secret,
                    'grant_type': 'authorization_code',
                    'redirect_uri': config.redirect_uri,
                    'code': this.authorizationCode
                };
                // refresh the token
            } else if (type === 'refresh') {
                data = {
                    'client_id': config.client_id,
                    'client_secret': config.client_secret,
                    'grant_type': 'refresh_token',
                    'refresh_token': this.token.refresh_token
                };
            }

            this.request.post(config.base_url + url, data, function (response) {
                window.storageManager.set('token', response);
                callback(response);
            });
        }
    }]);

    return OAuthToken;
}();
'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Request = function () {
    function Request() {
        _classCallCheck(this, Request);
    }

    /*
     * @param url: String
     * @param callback: Function
     *
     * @return void
     */


    _createClass(Request, [{
        key: 'get',
        value: function get(url, callback) {
            this.request('GET', url, {}, callback);
        }

        /*
         * @param url: String
         * @param callback: Function
         *
         * @return void
         */

    }, {
        key: 'put',
        value: function put(url, callback) {
            this.request('PUT', url, {}, callback);
        }

        /*
         * @param url: String
         * @param callback: Function
         *
         * @return void
         */

    }, {
        key: 'delete',
        value: function _delete(url, callback) {
            this.request('DELETE', url, {}, callback);
        }

        /*
         * @param url: String
         * @param callback: Function
         * @param data: Object
         *
         * @return void
         */

    }, {
        key: 'post',
        value: function post(url, data, callback) {
            this.request('POST', url, data, callback);
        }

        /*
         * @param type: String
         * @param url: String
         * @param data: Object
         * @param callback: Function
         *
         * @return void
         */

    }, {
        key: 'request',
        value: function request(type, url, data, callback) {
            var xmlhttp = new XMLHttpRequest();

            xmlhttp.onreadystatechange = function () {
                if (xmlhttp.readyState == XMLHttpRequest.DONE) {
                    callback(JSON.parse(xmlhttp.responseText));
                }
            };

            xmlhttp.open(type, url, true);

            if (type === 'POST') {
                xmlhttp.setRequestHeader("Content-type", "application/x-www-form-urlencoded");

                data = this.configurePostData(data);
                xmlhttp.send(data);
            } else {
                xmlhttp.send();
            }
        }

        /*
         * Encodes the data object to a string so that it can be sent as get parameters 
         *
         * @param data: Object
         *
         * @return: String
         */

    }, {
        key: 'configurePostData',
        value: function configurePostData(data) {
            var query = [];
            for (var key in data) {
                query.push(encodeURIComponent(key) + '=' + encodeURIComponent(data[key]));
            }

            data = query.join('&');

            return data;
        }
    }]);

    return Request;
}();
'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var SoundcloudSDK = function () {
    function SoundcloudSDK(callback) {
        _classCallCheck(this, SoundcloudSDK);

        this.baseUrl = 'https://api.soundcloud.com';
        this.baseUrlV2 = 'https://api-v2.soundcloud.com';

        this.request = new Request();

        this.oauthToken = new OAuthToken(function () {
            this.initializeApp(callback);
        }.bind(this));

        this.isPaginationRequestActive = false;
    }

    // Private


    _createClass(SoundcloudSDK, [{
        key: 'initializeApp',
        value: function initializeApp(callback) {
            this.getMe(function (response) {
                window.user = response;

                this.getLikedTrackIds('', [], function (likedTrackIds) {
                    window.dataManager.set('likedTrackIds', likedTrackIds);

                    this.getTrackRepostIds('', [], function (trackRepostIds) {
                        window.dataManager.set('trackRepostIds', trackRepostIds);

                        callback();
                    });
                }.bind(this));
            }.bind(this));
        }

        // Private

    }, {
        key: 'getMe',
        value: function getMe(callback) {
            var url = this.build_url('me', {}, {
                oauth_token: this.oauthToken.get('access_token')
            });

            this.request.get(this.baseUrlV2 + url, function (response) {
                callback(response);
            });
        }
    }, {
        key: 'getMyTracks',
        value: function getMyTracks(callback) {
            var myTracks = window.dataManager.get('myTracks');

            if (myTracks) {
                callback(myTracks);
                return;
            }

            var url = this.build_url('my_tracks', {}, {
                limit: 100,
                offset: 0,
                oauth_token: this.oauthToken.get('access_token')
            });

            this.request.get(this.baseUrl + url, function (myTracks) {
                window.dataManager.set('myTracks', myTracks);
                callback(myTracks);
            });
        }
    }, {
        key: 'getStream',
        value: function getStream(callback) {
            var stream = window.dataManager.get('stream');

            if (stream) {
                callback(stream);
                return;
            }

            var url = this.build_url('stream', {}, {
                limit: 100,
                oauth_token: this.oauthToken.get('access_token')
            });

            this.request.get(this.baseUrl + url, function (response) {
                config.soundcloud_urls['stream'].callback(response, callback);
            });
        }
    }, {
        key: 'getUserById',
        value: function getUserById(id, callback) {
            var url = this.build_url('user_by_id', { id: id }, {
                client_id: config.client_id
            });

            this.request.get(this.baseUrl + url, function (response) {
                callback(response);
            });
        }
    }, {
        key: 'getLikedTracks',
        value: function getLikedTracks(callback) {
            var liked_tracks = window.dataManager.get('liked_tracks');

            if (liked_tracks) {
                callback(liked_tracks);
                return;
            }

            var url = this.build_url('liked_tracks', { user_id: window.user.id }, {
                limit: 100,
                offset: 0,
                linked_partitioning: 1,
                oauth_token: this.oauthToken.get('access_token')
            });

            this.request.get(this.baseUrlV2 + url, function (response) {
                config.soundcloud_urls['liked_tracks'].callback(response, callback);
            });
        }
    }, {
        key: 'getLikedTrackIds',
        value: function getLikedTrackIds(url, array, callback) {
            var ids = window.dataManager.get('likedTrackIds');

            if (ids) {
                callback(ids);
                return;
            }

            if (url.length === 0) {
                url = this.baseUrl + this.build_url('liked_track_ids', {}, {
                    oauth_token: this.oauthToken.get('access_token'),
                    limit: 500,
                    linked_partitioning: 1,
                    page_number: 0,
                    page_size: 200
                });
            }

            this.request.get(url, function (response) {

                if (response.collection.length > 0) {
                    for (var i = 0; i < response.collection.length; i++) {
                        array.push(response.collection[i]);
                    }
                }

                if (response.hasOwnProperty('next_href')) {
                    this.getLikedTrackIds(response.next_href, array, callback);
                } else {
                    callback(array);
                }
            }.bind(this));
        }
    }, {
        key: 'getTrackRepostIds',
        value: function getTrackRepostIds(url, array, callback) {
            var ids = window.dataManager.get('trackRepostIds');

            if (ids) {
                callback(ids);
                return;
            }

            if (url.length === 0) {
                url = this.baseUrl + this.build_url('track_repost_ids', {}, {
                    oauth_token: this.oauthToken.get('access_token'),
                    limit: 5000,
                    linked_partitioning: 1,
                    page_number: 0,
                    page_size: 200
                });
            }

            this.request.get(url, function (response) {

                if (response.collection.length > 0) {
                    for (var i = 0; i < response.collection.length; i++) {
                        array.push(response.collection[i]);
                    }
                }

                if (response.hasOwnProperty('next_href')) {
                    this.getTrackRepostIds(response.next_href, array, callback);
                } else {
                    callback(array);
                }
            }.bind(this));
        }

        // Private

    }, {
        key: 'getPlaylists',
        value: function getPlaylists(callback) {
            var url = this.build_url('playlists', { user_id: window.user.id }, {
                oauth_token: this.oauthToken.get('access_token')
            });

            this.request.get(this.baseUrlV2 + url, function (response) {
                callback(response);
            });
        }
    }, {
        key: 'getOwnedPlaylists',
        value: function getOwnedPlaylists(callback) {
            var ownedPlaylists = window.dataManager.get('ownedPlaylists');

            if (ownedPlaylists) {
                callback(ownedPlaylists);
                return;
            }

            this.getPlaylists(function (response) {
                var playlists = response.collection;
                var ownedPlaylists = [];

                playlists.forEach(function (playlist) {
                    if (playlist.type === 'playlist') {
                        ownedPlaylists.push(playlist);
                    }
                });

                callback(ownedPlaylists);
            });
        }
    }, {
        key: 'getLikedPlaylists',
        value: function getLikedPlaylists(callback) {
            var likedPlaylists = window.dataManager.get('likedPlaylists');

            if (likedPlaylists) {
                callback(likedPlaylists);
                return;
            }

            this.getPlaylists(function (response) {
                var playlists = response.collection;
                var likedPlaylists = [];

                playlists.forEach(function (playlist) {
                    if (playlist.type === 'playlist-like') {
                        likedPlaylists.push(playlist);
                    }
                });

                callback(likedPlaylists);
            });
        }
    }, {
        key: 'getPlaylist',
        value: function getPlaylist(id, callback) {
            var url = this.build_url('playlist', { id: id }, {
                client_id: config.client_id
            });

            this.request.get(this.baseUrlV2 + url, function (response) {
                var trackIds = [];

                response.tracks.forEach(function (track) {
                    trackIds.push(track.id);
                });

                this.getTracksByIds(trackIds, callback);
            }.bind(this));
        }
    }, {
        key: 'getTracksByIds',
        value: function getTracksByIds(trackIds, callback) {
            var url = this.build_url('tracks_by_ids', {}, {
                ids: trackIds.join(),
                client_id: config.client_id
            });

            this.request.get(this.baseUrlV2 + url, function (response) {
                callback(response);
            });
        }
    }, {
        key: 'search',
        value: function search(query, callback) {
            var url = this.build_url('search', {}, {
                q: query,
                facet: 'genre',
                limit: 200,
                client_id: config.client_id,
                linked_partitioning: 1
            });

            this.request.get(this.baseUrlV2 + url, function (response) {
                var collection = response.collection;
                var tracks = [];

                collection.forEach(function (element) {
                    if (element.kind === 'track') {
                        tracks.push(element);
                    }
                });

                callback(tracks);
            });
        }
    }, {
        key: 'autocomplete',
        value: function autocomplete(query, callback) {
            var url = this.build_url('autocomplete', {}, {
                q: query,
                queries_limit: 0,
                results_limit: 20,
                client_id: config.client_id,
                limit: 20,
                offset: 0,
                linked_partitioning: 1
            });

            this.request.get(this.baseUrlV2 + url, function (response) {
                callback(response.results);
            });
        }
    }, {
        key: 'toggleLikedTrack',
        value: function toggleLikedTrack(track) {
            var url = '/users/' + window.user.id + '/favorites/' + track.id + '?oauth_token=' + this.oauthToken.get('access_token');

            if (window.dataManager.find('likedTrackIds', track.id)) {
                this.request.delete(this.baseUrl + url, function (response) {});
                // delete from trackIds and liked tracks
                window.dataManager.remove('likedTrackIds', track.id);
                window.dataManager.removeById('liked_tracks', track.id);
            } else {
                this.request.put(this.baseUrl + url, function (response) {});
                // add trackIds and liked tracks
                window.dataManager.unshift('likedTrackIds', track.id);
                window.dataManager.unshift('liked_tracks', track);
            }
        }
    }, {
        key: 'toggleRepostTrack',
        value: function toggleRepostTrack(id, callback) {
            var url = '/e1/me/track_reposts/' + id + '?oauth_token=' + this.oauthToken.get('access_token');

            if (window.dataManager.find('trackRepostIds', id)) {
                this.request.delete(this.baseUrlV2 + url, function (response) {});
                window.dataManager.remove('trackRepostIds', id);
            } else {
                this.request.put(this.baseUrlV2 + url, function (response) {});
                window.dataManager.unshift('trackRepostIds', id);
            }
        }
    }, {
        key: 'getTop50',
        value: function getTop50(kind, genre, callback) {
            genre = 'soundcloud:genres:' + genre;

            var url = this.build_url('top_50', {}, {
                kind: kind,
                genre: genre,
                client_id: config.client_id,
                limit: 200,
                offset: 0,
                linked_partitioning: 1
            });

            this.request.get(this.baseUrlV2 + url, function (response) {
                var tracks = [];

                response.collection.forEach(function (element) {
                    tracks.push(element.track);
                });

                callback(tracks);
            });
        }
    }, {
        key: 'getNotifications',
        value: function getNotifications(callback) {
            var url = this.build_url('notifications', {}, {
                limit: 5,
                offset: 0,
                linked_partitioning: 1,
                oauth_token: this.oauthToken.get('access_token')
            });

            this.request.get(this.baseUrlV2 + url, function (response) {
                callback(response.collection);
            });
        }
    }, {
        key: 'getPagination',
        value: function getPagination(type, callback) {
            if (this.isPaginationRequestActive) {
                return;
            }

            this.isPaginationRequestActive = true;

            var key = key = config.soundcloud_storage[type],
                url = this.append_auth_info(config.soundcloud_urls[key].next_href);

            if (url.indexOf('null') !== -1) {
                return;
            }

            this.request.get(url, function (response) {
                if (config.soundcloud_urls[key].callback !== undefined) {
                    config.soundcloud_urls[key].callback(response, callback);
                } else {
                    callback(response);
                }

                this.isPaginationRequestActive = false;
            }.bind(this));
        }
    }, {
        key: 'append_auth_info',
        value: function append_auth_info(url) {
            return url + '&client_id=' + config.client_id + '&oauth_token=' + this.oauthToken.get('access_token');
        }

        /******** URL BUILDING ********/

    }, {
        key: 'build_url',
        value: function build_url(type, base_function_params, params) {
            if (!this.test_params(type, params)) {
                console.error('Error building url: Mismatched parameters');
            }

            var url = void 0;

            if (Object.keys(base_function_params).length > 0) {
                url = this.generate_base_url(type, base_function_params) + '?';
            } else {
                url = config.soundcloud_urls[type].base + '?';
            }

            for (var key in params) {
                url += key + '=' + params[key] + '&';
            }

            return url;
        }
    }, {
        key: 'generate_base_url',
        value: function generate_base_url(type, params) {
            params = Object.keys(params).map(function (key) {
                return params[key];
            });
            return config.soundcloud_urls[type].base.apply(null, params);
        }
    }, {
        key: 'test_params',
        value: function test_params(type, params) {
            if (Object.keys(params).length !== config.soundcloud_urls[type].params.length) {
                return false;
            }

            for (var key in params) {
                if (config.soundcloud_urls[type].params.indexOf(key) === -1) {
                    return false;
                }
            }

            return true;
        }
    }]);

    return SoundcloudSDK;
}();